% Calculates scores for the "best K-basis" at each level of the tree
% (compare JTree_Basis)
%
% Usage:
%   [scores, levels] = JTree_Scores(X, K, Zpos,T,PCidx,maxlev,all_nodes, av_norm)
%
% Input:
%   X(N,dim)    data matrix
%   Zpos(J,3)  (UINT16) hierarchical cluster information; output from
%           "Build_JTree"; columns 1 and 2 contain the
%           positions of the components that are merged
%   T{J,1}  Element T{k} in the cell array contains a 2-by-2 matrix 
%           that decorrelates inputs X(:,Z(k,1)) and X(:,Z(k,2));
%           output from LKLB_Decorrelate.
%   PCidx(J,2)  (UINT8) id of principal components (1st or 2nd)
%   maxlev  default:dim-1
%
% Output:
%   scores, levels --- scores for the best K-basis
%
% Note:
%   Original high-res data x \in V0 = V1 + W1 = Vk + Wk + ... + W1
%   where V_k = V_{k+1} + W_{k+1}
%   basis=[sums(m-1,:); difs(m-1,:); ....; difs(1,:)] 
%
% By Ann Lee, CMU, April 2007. Last updated, ABL: 4/15/08

function [scores, levels] = JTree_Scores(X, K, Zpos,T,PCidx,maxlev,all_nodes, av_norm)

if size(Zpos,1)~=size(T,1), disp('Error in the dimensions of the inputs'); end
J=size(Zpos,1); % number of input variables
if nargin<4; 
    m=J+1; % dimension of the data
else
    m=size(all_nodes,2);  % dimension of the data
end
tmpfilts=[diag(ones(m,1))];   % each row is a component/band 
ind=cell(J,1);
if nargin<4
    maxlev = m-1;
end

% Initialization
sums=zeros(maxlev,m); difs=zeros(maxlev,m);
levels=(0:maxlev)';
scores=zeros(size(levels));
av_norm = mean(sum(X.*X,2));  % normalization term, E||x||^2
scores(1)= mean(sum(X(:,1:K).*X(:,1:K),2))/av_norm; % score at level L=0

for lev=1:maxlev  % loop through the levels
    s=tmpfilts(Zpos(lev,1:2),:);        % s(2,m); components to be merged
    R=T{lev};                      % R(2,2); rotation matrix
    y=R'*s; % updated: April 07
    tmpfilts(Zpos(lev,1:2),:)=y; 
    y=y(PCidx(lev,:),:); % y(1,:)=PC1, y(2,:)=PC2; sorted!
    sums(lev,:)=y(1,:);
    difs(lev,:)=y(2,:);
    nodes = all_nodes(lev,:); nodes=nodes(find(nodes>0)); 
    tmp = [diag(ones(m,1)); sums];    % see JTree_Basis
    basis = [tmp(nodes,:); flipud(difs(1:lev,:))]; % updated 
    
    % Score the "best K-basis"
    [avE, order, score]=Calc_Energy(X,basis,K,av_norm); 
    scores(lev+1)=score;
end